﻿--[[
Name: RatingBuster
Revision: $Revision: 32709 $
Developed by: Whitetooth@Cenarius (hotdogee@bahamut.twbbs.org)
Description: Converts combat ratings in tooltips into normal percentages.
]]

--------------
-- Librarys --
--------------
local TipHooker = AceLibrary("TipHooker-1.0")
local StatLogic = AceLibrary("StatLogic-1.0")
local L = AceLibrary("AceLocale-2.2"):new("RatingBuster")
local BC = AceLibrary("Babble-Class-2.2")


--------------------
-- AceAddon Setup --
--------------------
-- AceAddon Initialization
RatingBuster = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceConsole-2.0", "AceEvent-2.0", "AceDebug-2.0")
RatingBuster.title = "Rating Buster"
RatingBuster.version = "1.1.0 (r"..gsub("$Revision: 32709 $", "(%d+)", "%1")..")"
RatingBuster.date = gsub("$Date: 2007-04-15 08:04:07 -0400 (Sun, 15 Apr 2007) $", "^.-(%d%d%d%d%-%d%d%-%d%d).-$", "%1")


-------------------
-- Set Debugging --
-------------------
RatingBuster:SetDebugging(false)


-----------
-- Cache --
-----------
local cache = {}
setmetatable(cache, {__mode = "kv"}) -- weak table to enable garbage collection
local function clearCache()
	for k in pairs(cache) do
		cache[k] = nil
	end
end
--debug
--RatingBuster.cache = cache


---------------------
-- Local Variables --
---------------------
local _
local calcLevel, playerLevel
local profileDB, classDB -- Initialized in :OnInitialize()

-- Localize globals
local _G = getfenv(0)
local strfind = strfind
local strsub = strsub
local gsub = gsub
local pairs = pairs
local ipairs = ipairs
local type = type
local tinsert = tinsert
local tremove = tremove
local strsplit = strsplit
local strjoin = strjoin
local unpack = unpack
local tonumber = tonumber


---------------------
-- Saved Variables --
---------------------
-- Register DB
RatingBuster:RegisterDB("RatingBusterDB")
-- Default values
local profileDefault = {
	showItemLevel = true,
	showItemID = false,
	useRequiredLevel = false,
	customLevel = 0,
	textColor = {r = 1.0, g = 0.996,  b = 0.545, hex = "|cfffffe8b"},
	enableTextColor = true,
	enableStatMods = true,
	showRatings = true,
	defBreakDown = false,
	wpnBreakDown = false,
	showStats = true,
	showSum = true,
	showUnusableItem = false,
	showZeroValueStat = false,
	calcDiff = true,
	calcSum = true,
}
--[[
Str -> AP, Block, Healing
Agi -> Crit, Dodge, AP, RAP, Armor
Sta -> Health, SpellDmg
Int -> Mana, SpellCrit, SpellDmg, Healing, MP5, RAP, Armor
Spi -> MP5, MP5NC, HP5, SpellDmg, Healing
--]]
local classDefault = {
	-- Base stat conversions
	showAPFromStr = false,
	showBlockValueFromStr = false,
	showHealingFromStr = false, -- druid
	
	showCritFromAgi = true,
	showDodgeFromAgi = true,
	showAPFromAgi = false,
	showRAPFromAgi = false,
	showArmorFromAgi = false,
	
	showHealthFromSta = false,
	showSpellDmgFromSta = false, -- warlock
	
	showManaFromInt = false,
	showSpellCritFromInt = true,
	showSpellDmgFromInt = false, -- Druid, Mage, Paladin, Shaman, Warlock
	showHealingFromInt = false, -- Druid, Paladin, Shaman
	showMP5FromInt = false, -- Druid, Shaman
	showRAPFromInt = false, -- Hunter
	showArmorFromInt = false, -- Mage
	
	showMP5FromSpi = false, -- Druid, Mage, Priest
	showMP5NCFromSpi = false,
	showHP5FromSpi = false,
	showSpellDmgFromSpi = false, -- Priest
	showHealingFromSpi = false, -- Priest
	
	-- Stat Summary
	-- statbase
	sumHP = false,
	sumMP = false,
	sumAP = false,
	sumRAP = false,
	sumFAP = false,
	sumSpellDmg = false,
	sumArcaneDmg = false,
	sumFrostDmg = false,
	sumNatureDmg = false,
	sumFireDmg = false,
	sumShadowDmg = false,
	sumHolyDmg = false,
	sumHealing = false,
	sumHit = false,
	sumCrit = false,
	sumSpellHit = false,
	sumSpellCrit = false,
	sumDodge = false,
	sumParry = false,
	sumBlock = false,
	sumBlockValue = false,
	sumArmor = false,
	sumMP5 = false,
	sumHP5 = false,
	sumMP5NC = false,
	sumHP5OC = false,
	sumHitAvoid = false,
	sumCritAvoid = false,
	sumDodgeNeglect = false,
	sumParryNeglect = false,
	sumBlockNeglect = false,
	-- statcomposite
	sumStr = false,
	sumAgi = false,
	sumSta = false,
	sumInt = false,
	sumSpi = false,
	sumDefense = false,
	sumWeaponSkill = false,
}
local _, class = UnitClass("player")
if class == "DRUID" then
	classDefault.showHealingFromStr = true
end
if class == "PRIEST" or class == "MAGE" or class == "WARLOCK" then
	classDefault.showCritFromAgi = false
end
if class == "PRIEST" or class == "MAGE" or class == "WARLOCK" or class == "HUNTER" then
	classDefault.showDodgeFromAgi = false
end
if class == "WARLOCK" then
	classDefault.showSpellDmgFromSta = true
end
if class == "WARRIOR" or class == "ROGUE" or class == "HUNTER" then
	classDefault.showSpellCritFromInt = false
end
if class == "DRUID" or class == "MAGE" or class == "PALADIN" or class == "SHAMAN" or class == "WARLOCK" then
	classDefault.showSpellDmgFromInt = true
end
if class == "DRUID" or class == "PALADIN" or class == "SHAMAN" then
	classDefault.showHealingFromInt = true
end
if class == "DRUID" or class == "SHAMAN" then
	classDefault.showMP5FromInt = true
end
if class == "HUNTER" then
	classDefault.showRAPFromInt = true
end
if class == "MAGE" then
	classDefault.showArmorFromInt = true
end
if class == "DRUID" or class == "MAGE" or class == "PRIEST" then
	classDefault.showMP5FromSpi = true
end
if class == "PRIEST" then
	classDefault.showSpellDmgFromSpi = true
end
if class == "PRIEST" then
	classDefault.showHealingFromSpi = true
end
-- Stat Summary
if class == "WARRIOR" then
	classDefault.sumHP = true
	classDefault.sumAP = true
	classDefault.sumHit = true
	classDefault.sumCrit = true
end
if class == "PALADIN" then
	classDefault.sumHP = true
	classDefault.sumMP = true
	classDefault.sumHolyDmg = true
	classDefault.sumHealing = true
	classDefault.sumMP5 = true
end
if class == "HUNTER" then
	classDefault.sumRAP = true
	classDefault.sumHit = true
	classDefault.sumCrit = true
	classDefault.sumMP5 = true
end
if class == "ROGUE" then
	classDefault.sumHP = true
	classDefault.sumAP = true
	classDefault.sumHit = true
	classDefault.sumCrit = true
end
if class == "PRIEST" then
	classDefault.sumMP = true
	classDefault.sumSpellDmg = true
	classDefault.sumHealing = true
	classDefault.sumMP5 = true
end
if class == "SHAMAN" then
	classDefault.sumMP = true
	classDefault.sumSpellDmg = true
	classDefault.sumHealing = true
	classDefault.sumCrit = true
end
if class == "MAGE" then
	classDefault.sumHP = true
	classDefault.sumMP = true
	classDefault.sumSpellDmg = true
	classDefault.sumCrit = true
end
if class == "WARLOCK" then
	classDefault.sumHP = true
	classDefault.sumMP = true
	classDefault.sumSpellDmg = true
	classDefault.sumCrit = true
end
if class == "DRUID" then
	classDefault.sumHP = true
	classDefault.sumMP = true
	classDefault.sumFAP = true
	classDefault.sumSpellDmg = true
	classDefault.sumHealing = true
	classDefault.sumMP5 = true
	classDefault.sumDodge = true
	classDefault.sumArmor = true
end

-- Register Defaults
RatingBuster:RegisterDefaults("profile", profileDefault)
RatingBuster:RegisterDefaults("class", classDefault)


---------------------------
-- Slash Command Options --
---------------------------
local consoleOptions = {
	type = 'group',
	args = {
		statmod = {
			type = 'toggle',
			name = L["Enable Stat Mods"],
			desc = L["Enable support for Stat Mods"],
			get = function() return profileDB.enableStatMods end,
			set = function(v)
				profileDB.enableStatMods = v
				-- clear cache
				clearCache()
			end,
		},
		itemid = {
			type = 'toggle',
			name = L["Show ItemID"],
			desc = L["Show the ItemID in tooltips"],
			get = function() return profileDB.showItemID end,
			set = function(v)
				profileDB.showItemID = v
				-- clear cache
				clearCache()
			end,
		},
		itemlevel = {
			type = 'toggle',
			name = L["Show ItemLevel"],
			desc = L["Show the ItemLevel in tooltips"],
			get = function() return profileDB.showItemLevel end,
			set = function(v)
				profileDB.showItemLevel = v
				-- clear cache
				clearCache()
			end,
		},
		usereqlv = {
			type = 'toggle',
			name = L["Use required level"],
			desc = L["Calculate using the required level if you are below the required level"],
			get = function() return profileDB.useRequiredLevel end,
			set = function(v) profileDB.useRequiredLevel = v end,
		},
		level = {
			type = 'range',
			name = L["Set level"],
			desc = L["Set the level used in calculations (0 = your level)"],
			get = function() return profileDB.customLevel end,
			set = function(v) profileDB.customLevel = v end,
			min = 0,
			max = 73, -- set to level cap + 3
			step = 1,
		},
		rating = {
			type = 'group',
			name = L["Rating"],
			desc = L["Options for Rating display"],
			args = {
				show = {
					type = 'toggle',
					name = L["Show Rating conversions"],
					desc = L["Show Rating conversions in tooltips"],
					get = function() return profileDB.showRatings end,
					set = function(v)
						profileDB.showRatings = v
						-- clear cache
						clearCache()
					end,
				},
				def = {
					type = 'toggle',
					name = L["Defense breakdown"],
					desc = L["Convert Defense into Crit Avoidance, Hit Avoidance, Dodge, Parry and Block"],
					get = function() return profileDB.defBreakDown end,
					set = function(v)
						profileDB.defBreakDown = v
						-- clear cache
						clearCache()
					end,
				},
				wpn = {
					type = 'toggle',
					name = L["Weapon Skill breakdown"],
					desc = L["Convert Weapon Skill into Crit, Hit, Dodge Neglect, Parry Neglect, Block Neglect"],
					get = function() return profileDB.wpnBreakDown end,
					set = function(v)
						profileDB.wpnBreakDown = v
						-- clear cache
						clearCache()
					end,
				},
				color = {
					type = 'group',
					name = L["Change text color"],
					desc = L["Changes the color of added text"],
					args = {
						pick = {
							type = 'execute',
							name = L["Pick color"],
							desc = L["Pick a color"],
							func = function()
								CloseMenus()
								ColorPickerFrame.func = function()
									profileDB.textColor.r, profileDB.textColor.g, profileDB.textColor.b = ColorPickerFrame:GetColorRGB();
									profileDB.textColor.hex = "|cff"..string.format("%02x%02x%02x", profileDB.textColor.r * 255, profileDB.textColor.g * 255, profileDB.textColor.b * 255)
									-- clear cache
									clearCache()
								end
								ColorPickerFrame:SetColorRGB(profileDB.textColor.r, profileDB.textColor.g, profileDB.textColor.b);
								ColorPickerFrame.previousValues = {r = profileDB.textColor.r, g = profileDB.textColor.g, b = profileDB.textColor.b};
								ColorPickerFrame:Show()
							end,
						},
						enable = {
							type = 'toggle',
							name = L["Enable color"],
							desc = L["Enable colored text"],
							get = function() return profileDB.enableTextColor end,
							set = function(v)
								profileDB.enableTextColor = v
								-- clear cache
								clearCache()
							end,
						},
					},
				},
			},
		},
		stat = {
			type = 'group',
			name = L["Stat"],
			desc = L["Changes the display of base stats"],
			args = {
				show = {
					type = 'toggle',
					name = L["Show base stat conversions"],
					desc = L["Show base stat conversions in tooltips"],
					get = function() return profileDB.showStats end,
					set = function(v)
						profileDB.showStats = v
						-- clear cache
						clearCache()
					end,
				},
				str = {
					type = 'group',
					name = L["Strength"],
					desc = L["Changes the display of Strength"],
					args = {
						ap = {
							type = 'toggle',
							name = L["Show Attack Power"],
							desc = L["Show Attack Power from Strength"],
							get = function() return classDB.showAPFromStr end,
							set = function(v)
								classDB.showAPFromStr = v
								-- clear cache
								clearCache()
							end,
						},
						block = {
							type = 'toggle',
							name = L["Show Block Value"],
							desc = L["Show Block Value from Strength"],
							get = function() return classDB.showBlockValueFromStr end,
							set = function(v)
								classDB.showBlockValueFromStr = v
								-- clear cache
								clearCache()
							end,
						},
						heal = {
							type = 'toggle',
							name = L["Show Healing"].." ("..BC["Druid"]..")",
							desc = L["Show Healing from Strength"].." ("..BC["Druid"]..")",
							get = function() return classDB.showHealingFromStr end,
							set = function(v)
								classDB.showHealingFromStr = v
								-- clear cache
								clearCache()
							end,
						},
					},
				},
				agi = {
					type = 'group',
					name = L["Agility"],
					desc = L["Changes the display of Agility"],
					args = {
						crit = {
							type = 'toggle',
							name = L["Show Crit"],
							desc = L["Show Crit chance from Agility"],
							get = function() return classDB.showCritFromAgi end,
							set = function(v)
								classDB.showCritFromAgi = v
								-- clear cache
								clearCache()
							end,
						},
						dodge = {
							type = 'toggle',
							name = L["Show Dodge"],
							desc = L["Show Dodge chance from Agility"],
							get = function() return classDB.showDodgeFromAgi end,
							set = function(v)
								classDB.showDodgeFromAgi = v
								-- clear cache
								clearCache()
							end,
						},
						ap = {
							type = 'toggle',
							name = L["Show Attack Power"],
							desc = L["Show Attack Power from Agility"],
							get = function() return classDB.showAPFromAgi end,
							set = function(v)
								classDB.showAPFromAgi = v
								-- clear cache
								clearCache()
							end,
						},
						rap = {
							type = 'toggle',
							name = L["Show Ranged Attack Power"],
							desc = L["Show Ranged Attack Power from Agility"],
							get = function() return classDB.showRAPFromAgi end,
							set = function(v)
								classDB.showRAPFromAgi = v
								-- clear cache
								clearCache()
							end,
						},
						armor = {
							type = 'toggle',
							name = L["Show Armor"],
							desc = L["Show Armor from Agility"],
							get = function() return classDB.showArmorFromAgi end,
							set = function(v)
								classDB.showArmorFromAgi = v
								-- clear cache
								clearCache()
							end,
						},
					},
				},
				sta = {
					type = 'group',
					name = L["Stamina"],
					desc = L["Changes the display of Stamina"],
					args = {
						hp = {
							type = 'toggle',
							name = L["Show Health"],
							desc = L["Show Health from Stamina"],
							get = function() return classDB.showHealthFromSta end,
							set = function(v)
								classDB.showHealthFromSta = v
								-- clear cache
								clearCache()
							end,
						},
						dmg = {
							type = 'toggle',
							name = L["Show Spell Damage"].." ("..BC["Warlock"]..")",
							desc = L["Show Spell Damage from Stamina"].." ("..BC["Warlock"]..")",
							get = function() return classDB.showSpellDmgFromSta end,
							set = function(v)
								classDB.showSpellDmgFromSta = v
								-- clear cache
								clearCache()
							end,
						},
					},
				},
				int = {
					type = 'group',
					name = L["Intellect"],
					desc = L["Changes the display of Intellect"],
					args = {
						spellcrit = {
							type = 'toggle',
							name = L["Show Spell Crit"],
							desc = L["Show Spell Crit chance from Intellect"],
							get = function() return classDB.showSpellCritFromInt end,
							set = function(v)
								classDB.showSpellCritFromInt = v
								-- clear cache
								clearCache()
							end,
						},
						mp = {
							type = 'toggle',
							name = L["Show Mana"],
							desc = L["Show Mana from Intellect"],
							get = function() return classDB.showManaFromInt end,
							set = function(v)
								classDB.showManaFromInt = v
								-- clear cache
								clearCache()
							end,
						},
						dmg = {
							type = 'toggle',
							name = L["Show Spell Damage"].." ("..BC["Mage"]..", "..BC["Warlock"]..", "..BC["Druid"]..", "..BC["Shaman"]..", "..BC["Paladin"]..")",
							desc = L["Show Spell Damage from Intellect"].." ("..BC["Mage"]..", "..BC["Warlock"]..", "..BC["Druid"]..", "..BC["Shaman"]..", "..BC["Paladin"]..")",
							get = function() return classDB.showSpellDmgFromInt end,
							set = function(v)
								classDB.showSpellDmgFromInt = v
								-- clear cache
								clearCache()
							end,
						},
						heal = {
							type = 'toggle',
							name = L["Show Healing"].." ("..BC["Druid"]..", "..BC["Shaman"]..", "..BC["Paladin"]..")",
							desc = L["Show Healing from Intellect"].." ("..BC["Druid"]..", "..BC["Shaman"]..", "..BC["Paladin"]..")",
							get = function() return classDB.showHealingFromInt end,
							set = function(v)
								classDB.showHealingFromInt = v
								-- clear cache
								clearCache()
							end,
						},
						mp5 = {
							type = 'toggle',
							name = L["Show Mana Regen"].." ("..BC["Druid"]..", "..BC["Shaman"]..")",
							desc = L["Show Mana Regen while casting from Intellect"].." ("..BC["Druid"]..", "..BC["Shaman"]..")",
							get = function() return classDB.showMP5FromInt end,
							set = function(v)
								classDB.showMP5FromInt = v
								-- clear cache
								clearCache()
							end,
						},
						rap = {
							type = 'toggle',
							name = L["Show Ranged Attack Power"].." ("..BC["Hunter"]..")",
							desc = L["Show Ranged Attack Power from Intellect"].." ("..BC["Hunter"]..")",
							get = function() return classDB.showRAPFromInt end,
							set = function(v)
								classDB.showRAPFromInt = v
								-- clear cache
								clearCache()
							end,
						},
						armor = {
							type = 'toggle',
							name = L["Show Armor"].." ("..BC["Mage"]..")",
							desc = L["Show Armor from Intellect"].." ("..BC["Mage"]..")",
							get = function() return classDB.showArmorFromInt end,
							set = function(v)
								classDB.showArmorFromInt = v
								-- clear cache
								clearCache()
							end,
						},
					},
				},
				spi = {
					type = 'group',
					name = L["Spirit"],
					desc = L["Changes the display of Spirit"],
					args = {
						mp5 = {
							type = 'toggle',
							name = L["Show Mana Regen"].." ("..BC["Priest"]..", "..BC["Druid"]..", "..BC["Mage"]..")",
							desc = L["Show Mana Regen while casting from Spirit"].." ("..BC["Priest"]..", "..BC["Druid"]..", "..BC["Mage"]..")",
							get = function() return classDB.showMP5FromSpi end,
							set = function(v)
								classDB.showMP5FromSpi = v
								-- clear cache
								clearCache()
							end,
						},
						mp5nc = {
							type = 'toggle',
							name = L["Show Mana Regen (not casting)"],
							desc = L["Show Mana Regen while NOT casting from Spirit"],
							get = function() return classDB.showMP5NCFromSpi end,
							set = function(v)
								classDB.showMP5NCFromSpi = v
								-- clear cache
								clearCache()
							end,
						},
						hp5 = {
							type = 'toggle',
							name = L["Show Health Regen"],
							desc = L["Show Health Regen from Spirit"],
							get = function() return classDB.showHP5FromSpi end,
							set = function(v)
								classDB.showHP5FromSpi = v
								-- clear cache
								clearCache()
							end,
						},
						dmg = {
							type = 'toggle',
							name = L["Show Spell Damage"].." ("..BC["Priest"]..")",
							desc = L["Show Spell Damage from Spirit"].." ("..BC["Priest"]..")",
							get = function() return classDB.showSpellDmgFromSpi end,
							set = function(v)
								classDB.showSpellDmgFromSpi = v
								-- clear cache
								clearCache()
							end,
						},
						heal = {
							type = 'toggle',
							name = L["Show Healing"].." ("..BC["Priest"]..")",
							desc = L["Show Healing from Spirit"].." ("..BC["Priest"]..")",
							get = function() return classDB.showHealingFromSpi end,
							set = function(v)
								classDB.showHealingFromSpi = v
								-- clear cache
								clearCache()
							end,
						},
					},
				},
			},
		},
		sum = {
			type = 'group',
			name = L["Stat Summary"],
			desc = L["Options for stat summary"],
			args = {
				show = {
					type = 'toggle',
					name = L["Show stat summary"],
					desc = L["Show stat summary in tooltips"],
					get = function() return profileDB.showSum end,
					set = function(v)
						profileDB.showSum = v
						-- clear cache
						clearCache()
					end,
				},
				--[[
				showunusableitem = {
					type = 'toggle',
					name = L["Show summary for unusable item types"],
					desc = L["Show stat summary even for unusable item types"],
					get = function() return profileDB.showUnusableItem end,
					set = function(v)
						profileDB.showUnusableItem = v
						-- clear cache
						clearCache()
					end,
				},
				--]]
				showzerostat = {
					type = 'toggle',
					name = L["Show zero value stats"],
					desc = L["Show zero value stats in summary for consistancy"],
					get = function() return profileDB.showZeroValueStat end,
					set = function(v)
						profileDB.showZeroValueStat = v
						-- clear cache
						clearCache()
					end,
				},
				calcsum = {
					type = 'toggle',
					name = L["Calculate stat sum"],
					desc = L["Calculate the total stats for the item"],
					get = function() return profileDB.calcSum end,
					set = function(v)
						profileDB.calcSum = v
						-- clear cache
						clearCache()
					end,
				},
				calcdiff = {
					type = 'toggle',
					name = L["Calculate stat diff"],
					desc = L["Calculate the stat difference for the item and equipped items"],
					get = function() return profileDB.calcDiff end,
					set = function(v)
						profileDB.calcDiff = v
						-- clear cache
						clearCache()
					end,
				},
				stat = {
					type = 'group',
					name = L["Stat - Base"],
					desc = L["Choose base stats for summary"],
					args = {
						hp = {
							type = 'toggle',
							name = L["Sum Health"],
							desc = L["Health <- Health, Stamina"],
							get = function() return classDB.sumHP end,
							set = function(v)
								classDB.sumHP = v
								-- clear cache
								clearCache()
							end,
						},
						mp = {
							type = 'toggle',
							name = L["Sum Mana"],
							desc = L["Mana <- Mana, Intellect"],
							get = function() return classDB.sumMP end,
							set = function(v)
								classDB.sumMP = v
								-- clear cache
								clearCache()
							end,
						},
						ap = {
							type = 'toggle',
							name = L["Sum Attack Power"],
							desc = L["Attack Power <- Attack Power, Strength, Agility"],
							get = function() return classDB.sumAP end,
							set = function(v)
								classDB.sumAP = v
								-- clear cache
								clearCache()
							end,
						},
						rap = {
							type = 'toggle',
							name = L["Sum Ranged Attack Power"],
							desc = L["Ranged Attack Power <- Ranged Attack Power, Intellect, Attack Power, Strength, Agility"],
							get = function() return classDB.sumRAP end,
							set = function(v)
								classDB.sumRAP = v
								-- clear cache
								clearCache()
							end,
						},
						fap = {
							type = 'toggle',
							name = L["Sum Feral Attack Power"],
							desc = L["Feral Attack Power <- Feral Attack Power, Attack Power, Strength, Agility"],
							get = function() return classDB.sumFAP end,
							set = function(v)
								classDB.sumFAP = v
								-- clear cache
								clearCache()
							end,
						},
						dmg = {
							type = 'toggle',
							name = L["Sum Spell Damage"],
							desc = L["Spell Damage <- Spell Damage, Intellect, Spirit, Stamina"],
							get = function() return classDB.sumSpellDmg end,
							set = function(v)
								classDB.sumSpellDmg = v
								-- clear cache
								clearCache()
							end,
						},
						dmgholy = {
							type = 'toggle',
							name = L["Sum Holy Spell Damage"],
							desc = L["Holy Spell Damage <- Holy Spell Damage, Spell Damage, Intellect, Spirit"],
							get = function() return classDB.sumHolyDmg end,
							set = function(v)
								classDB.sumHolyDmg = v
								-- clear cache
								clearCache()
							end,
						},
						dmgarcane = {
							type = 'toggle',
							name = L["Sum Arcane Spell Damage"],
							desc = L["Arcane Spell Damage <- Arcane Spell Damage, Spell Damage, Intellect"],
							get = function() return classDB.sumArcaneDmg end,
							set = function(v)
								classDB.sumArcaneDmg = v
								-- clear cache
								clearCache()
							end,
						},
						dmgfire = {
							type = 'toggle',
							name = L["Sum Fire Spell Damage"],
							desc = L["Fire Spell Damage <- Fire Spell Damage, Spell Damage, Intellect, Stamina"],
							get = function() return classDB.sumFireDmg end,
							set = function(v)
								classDB.sumFireDmg = v
								-- clear cache
								clearCache()
							end,
						},
						dmgnature = {
							type = 'toggle',
							name = L["Sum Nature Spell Damage"],
							desc = L["Nature Spell Damage <- Nature Spell Damage, Spell Damage, Intellect"],
							get = function() return classDB.sumNatureDmg end,
							set = function(v)
								classDB.sumNatureDmg = v
								-- clear cache
								clearCache()
							end,
						},
						dmgfrost = {
							type = 'toggle',
							name = L["Sum Frost Spell Damage"],
							desc = L["Frost Spell Damage <- Frost Spell Damage, Spell Damage, Intellect"],
							get = function() return classDB.sumFrostDmg end,
							set = function(v)
								classDB.sumFrostDmg = v
								-- clear cache
								clearCache()
							end,
						},
						dmgshadow = {
							type = 'toggle',
							name = L["Sum Shadow Spell Damage"],
							desc = L["Shadow Spell Damage <- Shadow Spell Damage, Spell Damage, Intellect, Spirit, Stamina"],
							get = function() return classDB.sumShadowDmg end,
							set = function(v)
								classDB.sumShadowDmg = v
								-- clear cache
								clearCache()
							end,
						},
						heal = {
							type = 'toggle',
							name = L["Sum Healing"],
							desc = L["Healing <- Healing, Intellect, Spirit, Strength"],
							get = function() return classDB.sumHealing end,
							set = function(v)
								classDB.sumHealing = v
								-- clear cache
								clearCache()
							end,
						},
						hit = {
							type = 'toggle',
							name = L["Sum Hit Chance"],
							desc = L["Hit Chance <- Hit Rating, Weapon Skill Rating"],
							get = function() return classDB.sumHit end,
							set = function(v)
								classDB.sumHit = v
								-- clear cache
								clearCache()
							end,
						},
						hitspell = {
							type = 'toggle',
							name = L["Sum Spell Hit Chance"],
							desc = L["Spell Hit Chance <- Spell Hit Rating"],
							get = function() return classDB.sumSpellHit end,
							set = function(v)
								classDB.sumSpellHit = v
								-- clear cache
								clearCache()
							end,
						},
						crit = {
							type = 'toggle',
							name = L["Sum Crit Chance"],
							desc = L["Crit Chance <- Crit Rating, Agility, Weapon Skill Rating"],
							get = function() return classDB.sumCrit end,
							set = function(v)
								classDB.sumCrit = v
								-- clear cache
								clearCache()
							end,
						},
						critspell = {
							type = 'toggle',
							name = L["Sum Spell Crit Chance"],
							desc = L["Spell Crit Chance <- Spell Crit Rating, Intellect"],
							get = function() return classDB.sumSpellCrit end,
							set = function(v)
								classDB.sumSpellCrit = v
								-- clear cache
								clearCache()
							end,
						},
						mp5 = {
							type = 'toggle',
							name = L["Sum Mana Regen"],
							desc = L["Mana Regen <- Mana Regen, Spirit"],
							get = function() return classDB.sumMP5 end,
							set = function(v)
								classDB.sumMP5 = v
								-- clear cache
								clearCache()
							end,
						},
						mp5nc = {
							type = 'toggle',
							name = L["Sum Mana Regen while not casting"],
							desc = L["Mana Regen while not casting <- Spirit"],
							get = function() return classDB.sumMP5NC end,
							set = function(v)
								classDB.sumMP5NC = v
								-- clear cache
								clearCache()
							end,
						},
						hp5 = {
							type = 'toggle',
							name = L["Sum Health Regen"],
							desc = L["Health Regen <- Health Regen"],
							get = function() return classDB.sumHP5 end,
							set = function(v)
								classDB.sumHP5 = v
								-- clear cache
								clearCache()
							end,
						},
						hp5oc = {
							type = 'toggle',
							name = L["Sum Health Regen when out of combat"],
							desc = L["Health Regen when out of combat <- Spirit"],
							get = function() return classDB.sumHP5OC end,
							set = function(v)
								classDB.sumHP5OC = v
								-- clear cache
								clearCache()
							end,
						},
						armor = {
							type = 'toggle',
							name = L["Sum Armor"],
							desc = L["Armor <- Armor from items, Armor from bonuses, Agility, Intellect"],
							get = function() return classDB.sumArmor end,
							set = function(v)
								classDB.sumArmor = v
								-- clear cache
								clearCache()
							end,
						},
						blockvalue = {
							type = 'toggle',
							name = L["Sum Block Value"],
							desc = L["Block Value <- Block Value, Strength"],
							get = function() return classDB.sumBlockValue end,
							set = function(v)
								classDB.sumBlockValue = v
								-- clear cache
								clearCache()
							end,
						},
						dodge = {
							type = 'toggle',
							name = L["Sum Dodge Chance"],
							desc = L["Dodge Chance <- Dodge Rating, Agility, Defense Rating"],
							get = function() return classDB.sumDodge end,
							set = function(v)
								classDB.sumDodge = v
								-- clear cache
								clearCache()
							end,
						},
						parry = {
							type = 'toggle',
							name = L["Sum Parry Chance"],
							desc = L["Parry Chance <- Parry Rating, Defense Rating"],
							get = function() return classDB.sumParry end,
							set = function(v)
								classDB.sumParry = v
								-- clear cache
								clearCache()
							end,
						},
						block = {
							type = 'toggle',
							name = L["Sum Block Chance"],
							desc = L["Block Chance <- Block Rating, Defense Rating"],
							get = function() return classDB.sumBlock end,
							set = function(v)
								classDB.sumBlock = v
								-- clear cache
								clearCache()
							end,
						},
						avoidhit = {
							type = 'toggle',
							name = L["Sum Hit Avoidance"],
							desc = L["Hit Avoidance <- Defense Rating"],
							get = function() return classDB.sumHitAvoid end,
							set = function(v)
								classDB.sumHitAvoid = v
								-- clear cache
								clearCache()
							end,
						},
						avoidcrit = {
							type = 'toggle',
							name = L["Sum Crit Avoidance"],
							desc = L["Crit Avoidance <- Defense Rating, Resilience"],
							get = function() return classDB.sumCritAvoid end,
							set = function(v)
								classDB.sumCritAvoid = v
								-- clear cache
								clearCache()
							end,
						},
						neglectdodge = {
							type = 'toggle',
							name = L["Sum Dodge Neglect"],
							desc = L["Dodge Neglect <- Weapon Skill Rating"],
							get = function() return classDB.sumDodgeNeglect end,
							set = function(v)
								classDB.sumDodgeNeglect = v
								-- clear cache
								clearCache()
							end,
						},
						neglectparry = {
							type = 'toggle',
							name = L["Sum Parry Neglect"],
							desc = L["Parry Neglect <- Weapon Skill Rating"],
							get = function() return classDB.sumParryNeglect end,
							set = function(v)
								classDB.sumParryNeglect = v
								-- clear cache
								clearCache()
							end,
						},
						neglectblock = {
							type = 'toggle',
							name = L["Sum Block Neglect"],
							desc = L["Block Neglect <- Weapon Skill Rating"],
							get = function() return classDB.sumBlockNeglect end,
							set = function(v)
								classDB.sumBlockNeglect = v
								-- clear cache
								clearCache()
							end,
						},
					},
				},
				statcomp = {
					type = 'group',
					name = L["Stat - Composite"],
					desc = L["Choose composite stats for summary"],
					args = {
						str = {
							type = 'toggle',
							name = L["Sum Strength"],
							desc = L["Strength Summary"],
							get = function() return classDB.sumStr end,
							set = function(v)
								classDB.sumStr = v
								-- clear cache
								clearCache()
							end,
						},
						agi = {
							type = 'toggle',
							name = L["Sum Agility"],
							desc = L["Agility Summary"],
							get = function() return classDB.sumAgi end,
							set = function(v)
								classDB.sumAgi = v
								-- clear cache
								clearCache()
							end,
						},
						sta = {
							type = 'toggle',
							name = L["Sum Stamina"],
							desc = L["Stamina Summary"],
							get = function() return classDB.sumSta end,
							set = function(v)
								classDB.sumSta = v
								-- clear cache
								clearCache()
							end,
						},
						int = {
							type = 'toggle',
							name = L["Sum Intellect"],
							desc = L["Intellect Summary"],
							get = function() return classDB.sumInt end,
							set = function(v)
								classDB.sumInt = v
								-- clear cache
								clearCache()
							end,
						},
						spi = {
							type = 'toggle',
							name = L["Sum Spirit"],
							desc = L["Spirit Summary"],
							get = function() return classDB.sumSpi end,
							set = function(v)
								classDB.sumSpi = v
								-- clear cache
								clearCache()
							end,
						},
						def = {
							type = 'toggle',
							name = L["Sum Defense"],
							desc = L["Defense <- Defense Rating"],
							get = function() return classDB.sumDefense end,
							set = function(v)
								classDB.sumDefense = v
								-- clear cache
								clearCache()
							end,
						},
						wpn = {
							type = 'toggle',
							name = L["Sum Weapon Skill"],
							desc = L["Weapon Skill <- Weapon Skill Rating"],
							get = function() return classDB.sumWeaponSkill end,
							set = function(v)
								classDB.sumWeaponSkill = v
								-- clear cache
								clearCache()
							end,
						},
					},
				},
			},
		},
	},
}

RatingBuster:RegisterChatCommand({"/rb", "/ratingbuster"}, consoleOptions)


-----------
-- Tools --
-----------
-- copyTable
local function copyTable(to, from)
	if to then
		for k in pairs(to) do
			to[k] = nil
		end
		setmetatable(to, nil)
	else
		to = {}
	end
	for k,v in pairs(from) do
		if type(k) == "table" then
			k = copyTable({}, k)
		end
		if type(v) == "table" then
			v = copyTable({}, v)
		end
		to[k] = v
	end
	setmetatable(to, getmetatable(from))
	return to
end


---------------------
-- Initializations --
---------------------
--[[ Loading Process Event Reference
{
ADDON_LOADED - When this addon is loaded
VARIABLES_LOADED - When all addons are loaded
PLAYER_LOGIN - Most information about the game world should now be available to the UI
}
--]]
-- OnInitialize(name) called at ADDON_LOADED
function RatingBuster:OnInitialize()
	profileDB = RatingBuster.db.profile
	classDB = RatingBuster.db.class
end

-- OnEnable() called at PLAYER_LOGIN
function RatingBuster:OnEnable()
	-- Hook item tooltips
	TipHooker:Hook(self.ProcessTooltip, "item")
	-- Initialize playerLevel
	playerLevel = UnitLevel("player")
	-- for setting a new level
	self:RegisterEvent("PLAYER_LEVEL_UP")
	-- Events that require cache clearing
	self:RegisterEvent("CHARACTER_POINTS_CHANGED", clearCache) -- talent point changed
	self:RegisterEvent("UNIT_AURA", 1) -- fire at most once every 1 second
end

function RatingBuster:OnDisable()
	-- Unhook item tooltips
	TipHooker:Unhook(self.ProcessTooltip, "item")
end

-- event = PLAYER_LEVEL_UP
-- arg1 = New player level
function RatingBuster:PLAYER_LEVEL_UP(newlevel)
	playerLevel = newlevel
	clearCache()
end

-- event = UNIT_AURA
-- arg1 = the UnitID of the entity
function RatingBuster:UNIT_AURA(unit)
	if (unit ~= "player") then return end
	clearCache()
end

--------------------------
-- Process Tooltip Core --
--------------------------
--[[
"+15 Agility"
-> "+15 Agility (+0.46% Crit)"
"+15 Crit Rating"
-> "+15 Crit Rating (+1.20%)"
"Equip: Increases your hit rating by 10."
-> "Equip: Increases your hit rating by 10 (+1.20%)."
--]]
function RatingBuster.ProcessTooltip(tooltip, name, link)
	-- Check if we're in standby mode
	--if not RatingBuster:IsActive() then return end
	---------------------------
	-- Set calculation level --
	---------------------------
	calcLevel = profileDB.customLevel
	if calcLevel == 0 then
		calcLevel = playerLevel
	end
	if profileDB.useRequiredLevel and link then
		local _, _, _, _, reqLevel = GetItemInfo(link)
		--RatingBuster:Print(link..", "..calcLevel)
		if reqLevel and calcLevel < reqLevel then
			calcLevel = reqLevel
		end
	end
	---------------------
	-- Tooltip Scanner --
	---------------------
	-- Loop through tooltip lines starting at line 2
	local tipTextLeft = tooltip:GetName().."TextLeft"
	for i = 2, tooltip:NumLines() do
		local fontString = _G[tipTextLeft..i]
		local text = fontString:GetText()
		if text then -- do nothing if we don't find a number
			-- Get data from cache if available
			local cacheID = text..calcLevel
			local cacheText = cache[cacheID]
			if cacheText then
				if cacheText ~= text then
					fontString:SetText(cacheText)
				end
			elseif strfind(text, "%d") then
				-- Initial pattern check, do nothing if not found
				-- Check for separators and bulid separatorTable
				local separatorTable = {}
				for _, sep in ipairs(L["separators"]) do
					if strfind(text, sep) then
						tinsert(separatorTable, sep)
					end
				end
				-- SplitDoJoin
				text = RatingBuster:SplitDoJoin(text, separatorTable)
				cache[cacheID] = text
				-- SetText
				fontString:SetText(text)
			end
		end
	end
	----------------------------
	-- Item Level and Item ID --
	----------------------------
	-- Check for ItemLevel addon, do nothing if found
	if not ItemLevel_AddInfo and (profileDB.showItemLevel or profileDB.showItemID) and link then
		if cache[link] then
			tooltip:AddLine(cache[link])
		else
			-- Get the Item ID from the link string
			local _, link, _, level = GetItemInfo(link)
			local _, _, id = strfind(link, "item:(%d+)")
			local newLine = ""
			if level and profileDB.showItemLevel then
				newLine = newLine..L["ItemLevel: "]..level
			end
			if id and profileDB.showItemID then
				if newLine ~= "" then
					newLine = newLine..", "
				end
				newLine = newLine..L["ItemID: "]..id
			end
			if newLine ~= "" then
				cache[link] = newLine
				tooltip:AddLine(newLine)
			end
		end
	end
	------------------
	-- Stat Summary --
	------------------
	--[[
	-- Base Stats
	-- Health - HEALTH, STA
	-- Mana - MANA, INT
	-- Attack Power - AP, STR, AGI
	-- Ranged Attack Power - RANGED_AP, INT, AP, STR, AGI
	-- Feral Attack Power - FERAL_AP, AP, STR, AGI
	-- Spell Damage - SPELL_DMG, STA, INT, SPI
	-- Holy Damage - HOLY_SPELL_DMG, SPELL_DMG, INT, SPI
	-- Arcane Damage - ARCANE_SPELL_DMG, SPELL_DMG, INT
	-- Fire Damage - FIRE_SPELL_DMG, SPELL_DMG, STA, INT
	-- Nature Damage - NATURE_SPELL_DMG, SPELL_DMG, INT
	-- Frost Damage - FROST_SPELL_DMG, SPELL_DMG, INT
	-- Shadow Damage - SHADOW_SPELL_DMG, SPELL_DMG, STA, INT, SPI
	-- Healing - HEAL, STR, INT, SPI
	-- Hit Chance - MELEE_HIT_RATING, WEAPON_RATING
	-- Crit Chance - MELEE_CRIT_RATING, WEAPON_RATING, AGI
	-- Spell Hit Chance - SPELL_HIT_RATING
	-- Spell Crit Chance - SPELL_CRIT_RATING, INT
	-- Mana Regen - MANA_REG, SPI
	-- Health Regen - HEALTH_REG
	-- Mana Regen Not Casting - SPI
	-- Health Regen While Casting - SPI
	-- Armor - ARMOR, ARMOR_BONUS, AGI, INT
	-- Block Value - BLOCK_VALUE, STR
	-- Dodge Chance - DODGE_RATING, DEFENSE_RATING, AGI
	-- Parry Chance - PARRY_RATING, DEFENSE_RATING
	-- Block Chance - BLOCK_RATING, DEFENSE_RATING
	-- Hit Avoidance - DEFENSE_RATING, MELEE_HIT_AVOID_RATING
	-- Crit Avoidance - DEFENSE_RATING, RESILIENCE_RATING, MELEE_CRIT_AVOID_RATING
	-- Dodge Neglect - WEAPON_RATING
	-- Parry Neglect - WEAPON_RATING
	-- Block Neglect - WEAPON_RATING
	-- Composite Stats
	-- Strength - STR
	-- Agility - AGI
	-- Stamina - STA
	-- Intellect - INT
	-- Spirit - SPI
	-- Defense - DEFENSE_RATING
	-- Weapon Skill - WEAPON_RATING
	--]]
	if profileDB.showSum then
		RatingBuster:StatSummary(tooltip, name, link)
	end
	---------------------
	-- Repaint tooltip --
	---------------------
	tooltip:Show()
end

---------------------------------------------------------------------------------
-- Recursive algorithm that divides a string into pieces using the separators in separatorTable,
-- processes them separately, then joins them back together
---------------------------------------------------------------------------------
-- text = "+24 Agility/+4 Stamina and +4 Spell Crit/+5 Spirit"
-- separatorTable = {"/", " and ", ","}
-- RatingBuster:SplitDoJoin("+24 Agility/+4 Stamina, +4 Dodge and +4 Spell Crit/+5 Spirit", {"/", " and ", ","})
--
function RatingBuster:SplitDoJoin(text, separatorTable)
	if type(separatorTable) == "table" and table.maxn(separatorTable) > 0 then
		local sep = tremove(separatorTable, 1)
		text =  gsub(text, sep, "@")
		text = {strsplit("@", text)}
		local processedText = {}
		local tempTable = {}
		for _, t in ipairs(text) do
			--self:Print(t[1])
			copyTable(tempTable, separatorTable)
			tinsert(processedText, self:SplitDoJoin(t, tempTable))
		end
		-- Join text
		return (gsub(strjoin("@", unpack(processedText)), "@", sep))
	else
		--self:Print(cacheID)
		return self:ProcessText(text)
	end
end


function RatingBuster:ProcessText(text)
	--self:Print(text)
	-- Check if test has a matching pattern
	for _, num in ipairs(L["numberPatterns"]) do
		-- Convert text to lower so we don't have to worry about same ratings with different cases
		local lowerText = string.lower(text)
		-- Capture the stat value
		local s, e, value = strfind(lowerText, num.pattern)
		if value then
			-- Capture the stat name
			for _, stat in ipairs(L["statList"]) do
				if strfind(lowerText, stat.pattern) then
					value = tonumber(value)
					local infoString = ""
					if type(stat.id) == "number" and stat.id >= 1 and stat.id <= 23 and profileDB.showRatings then
						--------------------
						-- Combat Ratings --
						--------------------
						-- Calculate stat value
						local effect, strID = StatLogic:GetEffectFromRating(value, stat.id, calcLevel)
						--self:Debug(reversedAmount..", "..amount..", "..v[2]..", "..RatingBuster.targetLevel)-- debug
						-- If rating is resilience, add a minus sign
						if stat.id >= 15 and stat.id <= 17 then
							effect = effect * -1
						end
						if strID == "DEFENSE" and profileDB.defBreakDown then
							effect = effect * 0.04
							local numStats = 5
							if GetParryChance() == 0 then
								numStats = numStats - 1
							end
							if GetBlockChance() == 0 then
								numStats = numStats - 1
							end
							infoString = format("%+.2f%% x"..numStats, effect)
						elseif strID == "WEAPON_SKILL" and profileDB.wpnBreakDown then
							effect = effect * 0.04
							infoString = format("%+.2f%% x5", effect)
						else
							--self:Debug(text..", "..tostring(effect)..", "..value..", "..stat.id..", "..calcLevel)
							-- Build info string
							infoString = format("%+.2f", effect)
							if stat.id > 2 and stat.id < 21 then -- if rating is a percentage
								infoString = infoString.."%"
							end
						end
					elseif stat.id == SPELL_STAT1_NAME and profileDB.showStats then
						--------------
						-- Strength --
						--------------
						local statmod = 1
						if profileDB.enableStatMods then
							statmod = StatLogic:GetStatMod("MOD_STR")
							value = value * statmod
						end
						local infoTable = {}
						if classDB.showAPFromStr then
							local mod = StatLogic:GetStatMod("MOD_AP")
							local effect = value * StatLogic:GetAPPerStr(class) * mod
							if (mod ~= 1 or statmod ~= 1) and floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value AP"], "$value", format("%+.1f", effect))))
							elseif floor(effect + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value AP"], "$value", format("%+.0f", effect))))
							end
						end
						if classDB.showBlockValueFromStr then
							local effect = value * StatLogic:GetBlockValuePerStr(class)
							if floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value Block"], "$value", format("%+.1f", effect))))
							end
						end
						if classDB.showHealingFromStr then
							local mod = StatLogic:GetStatMod("MOD_HEALING")
							local effect = value * StatLogic:GetStatMod("ADD_HEALING_MOD_STR") * mod
							if floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value Heal"], "$value", format("%+.1f", effect))))
							end
						end
						infoString = strjoin(", ", unpack(infoTable))
					elseif stat.id == SPELL_STAT2_NAME and profileDB.showStats then
						-------------
						-- Agility --
						-------------
						local statmod = 1
						if profileDB.enableStatMods then
							statmod = StatLogic:GetStatMod("MOD_AGI")
							value = value * statmod
						end
						local infoTable = {}
						if classDB.showAPFromAgi then
							local mod = StatLogic:GetStatMod("MOD_AP")
							local effect = value * StatLogic:GetAPPerAgi(class) * mod
							if (mod ~= 1 or statmod ~= 1) and floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value AP"], "$value", format("%+.1f", effect))))
							elseif floor(effect + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value AP"], "$value", format("%+.0f", effect))))
							end
						end
						if classDB.showRAPFromAgi then
							local mod = StatLogic:GetStatMod("MOD_RANGED_AP")
							local effect = value * StatLogic:GetRAPPerAgi(class) * mod
							if (mod ~= 1 or statmod ~= 1) and floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value RAP"], "$value", format("%+.1f", effect))))
							elseif floor(effect + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value RAP"], "$value", format("%+.0f", effect))))
							end
						end
						if classDB.showCritFromAgi then
							local effect = StatLogic:GetCritFromAgi(value, class, calcLevel)
							if effect > 0 then
								tinsert(infoTable, (gsub(L["$value% Crit"], "$value", format("%+.2f", effect))))
							end
						end
						if classDB.showDodgeFromAgi and (calcLevel == playerLevel) then
							local effect = StatLogic:GetDodgeFromAgi(value)
							if effect > 0 then
								tinsert(infoTable, (gsub(L["$value% Dodge"], "$value", format("%+.2f", effect))))
							end
						end
						if classDB.showArmorFromAgi then
							local effect = value * 2
							if effect > 0 then
								tinsert(infoTable, (gsub(L["$value Armor"], "$value", format("%+.0f", effect))))
							end
						end
						infoString = strjoin(", ", unpack(infoTable))
					elseif stat.id == SPELL_STAT3_NAME and profileDB.showStats then
						-------------
						-- Stamina --
						-------------
						local statmod = 1
						if profileDB.enableStatMods then
							statmod = StatLogic:GetStatMod("MOD_STA")
							value = value * statmod
						end
						local infoTable = {}
						if classDB.showHealthFromSta then
							local mod = StatLogic:GetStatMod("MOD_HEALTH")
							local effect = value * 10 * mod -- 10 Health per Sta
							if (mod ~= 1 or statmod ~= 1) and floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value HP"], "$value", format("%+.1f", effect))))
							elseif floor(effect + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value HP"], "$value", format("%+.0f", effect))))
							end
						end
						if classDB.showSpellDmgFromSta then
							local mod = StatLogic:GetStatMod("MOD_SPELL_DMG")
							local effect = value * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA") * mod
							if floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value Dmg"], "$value", format("%+.1f", effect))))
							end
						end
						infoString = strjoin(", ", unpack(infoTable))
					elseif stat.id == SPELL_STAT4_NAME and profileDB.showStats then
						---------------
						-- Intellect --
						---------------
						local statmod = 1
						if profileDB.enableStatMods then
							statmod = StatLogic:GetStatMod("MOD_INT")
							value = value * statmod
						end
						local infoTable = {}
						if classDB.showManaFromInt then
							local mod = StatLogic:GetStatMod("MOD_MANA")
							local effect = value * 15 * mod -- 15 Mana per Int
							if (mod ~= 1 or statmod ~= 1) and floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value MP"], "$value", format("%+.1f", effect))))
							elseif floor(effect + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value MP"], "$value", format("%+.0f", effect))))
							end
						end
						if classDB.showSpellCritFromInt then
							local effect = StatLogic:GetSpellCritFromInt(value, class, calcLevel)
							if effect > 0 then
								tinsert(infoTable, (gsub(L["$value% Spell Crit"], "$value", format("%+.2f", effect))))
							end
						end
						if classDB.showSpellDmgFromInt then
							local mod = StatLogic:GetStatMod("MOD_SPELL_DMG")
							local effect = value * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT") * mod
							if floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value Dmg"], "$value", format("%+.1f", effect))))
							end
						end
						if classDB.showHealingFromInt then
							local mod = StatLogic:GetStatMod("MOD_HEALING")
							local effect = value * StatLogic:GetStatMod("ADD_HEALING_MOD_INT") * mod
							if floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value Heal"], "$value", format("%+.1f", effect))))
							end
						end
						if classDB.showMP5FromInt then
							local effect = value * StatLogic:GetStatMod("ADD_MANA_REG_MOD_INT")
							if floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value MP5"], "$value", format("%+.1f", effect))))
							end
						end
						if classDB.showRAPFromInt then
							local mod = StatLogic:GetStatMod("MOD_RANGED_AP")
							local effect = value * StatLogic:GetStatMod("ADD_RANGED_AP_MOD_INT") * mod
							if floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value RAP"], "$value", format("%+.1f", effect))))
							end
						end
						if classDB.showArmorFromInt then
							local effect = value * StatLogic:GetStatMod("ADD_ARMOR_MOD_INT")
							if floor(effect + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value Armor"], "$value", format("%+.0f", effect))))
							end
						end
						infoString = strjoin(", ", unpack(infoTable))
					elseif stat.id == SPELL_STAT5_NAME and profileDB.showStats then
						------------
						-- Spirit --
						------------
						local statmod = 1
						if profileDB.enableStatMods then
							statmod = StatLogic:GetStatMod("MOD_SPI")
							value = value * statmod
						end
						local infoTable = {}
						if classDB.showMP5FromSpi then
							local mod = StatLogic:GetStatMod("ADD_MANA_REG_MOD_NORMAL_MANA_REG")
							local effect = StatLogic:GetNormalManaRegenFromSpi(value, class) * mod
							if floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value MP5"], "$value", format("%+.1f", effect))))
							end
						end
						if classDB.showMP5NCFromSpi then
							local effect = StatLogic:GetNormalManaRegenFromSpi(value, class)
							if floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value MP5(NC)"], "$value", format("%+.1f", effect))))
							end
						end
						if classDB.showHP5FromSpi then
							local effect = StatLogic:GetHealthRegenFromSpi(value, class)
							if floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value HP5"], "$value", format("%+.1f", effect))))
							end
						end
						if classDB.showSpellDmgFromSpi then
							local mod = StatLogic:GetStatMod("MOD_SPELL_DMG")
							local effect = value * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI") * mod
							if floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value Dmg"], "$value", format("%+.1f", effect))))
							end
						end
						if classDB.showHealingFromSpi then
							local mod = StatLogic:GetStatMod("MOD_HEALING")
							local effect = value * StatLogic:GetStatMod("ADD_HEALING_MOD_SPI") * mod
							if floor(effect * 10 + 0.5) > 0 then
								tinsert(infoTable, (gsub(L["$value Heal"], "$value", format("%+.1f", effect))))
							end
						end
						infoString = strjoin(", ", unpack(infoTable))
					end
					if infoString ~= "" then
						-- Add parenthesis
						infoString = "("..infoString..")"
						-- Add Color
						if profileDB.enableTextColor then
							infoString = profileDB.textColor.hex..infoString.."|r"
						end
						-- Build replacement string
						if num.addInfo == "AfterNumber" then -- Add after number
							infoString = gsub(infoString, "%%", "%%%%%%%%")
							infoString = gsub(strsub(text, s, e), "%d+", "%0 "..infoString)
						else -- Add after stat
							infoString = gsub(infoString, "%%", "%%%%")
							s, e = strfind(lowerText, stat.pattern)
							infoString = "%0 "..infoString
						end
						-- Insert info into text
						return (gsub(text, strsub(text, s, e), infoString, 1)) -- because gsub has 2 return values, but we only want 1
					end
					return text
				end
			end
		end
	end
	return text
end


-- Color Numbers
local GREEN_FONT_COLOR_CODE = GREEN_FONT_COLOR_CODE -- "|cff20ff20" Green
local HIGHLIGHT_FONT_COLOR_CODE = HIGHLIGHT_FONT_COLOR_CODE -- "|cffffffff" White
local RED_FONT_COLOR_CODE = RED_FONT_COLOR_CODE -- "|cffffffff" White
local FONT_COLOR_CODE_CLOSE = FONT_COLOR_CODE_CLOSE -- "|r"
local function colorNum(text, num)
	if num > 0 then
		return GREEN_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE
	elseif num < 0 then
		return RED_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE
	else
		return HIGHLIGHT_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE
	end
end


function RatingBuster:StatSummary(tooltip, name, link)
	-- Check Cache
	local id = StatLogic:GetDiffID(link)
	if cache[id] then
		if table.maxn(cache[id]) == 0 then return end
		-- Write Tooltip
		tooltip:AddLine(HIGHLIGHT_FONT_COLOR_CODE..L["Stat Summary"]..FONT_COLOR_CODE_CLOSE)
		tooltip:AddTexture("Interface\\AddOns\\RatingBuster\\images\\Sigma")
		for _, o in ipairs(cache[id]) do
			tooltip:AddDoubleLine(o[1], o[2])
		end
		return
	end
	-------------------------
	-- Build Summary Table --
	local sum = StatLogic:GetSum(link)
	if not sum then return end
	local diff1, diff2 = StatLogic:GetDiff(link)
	-- Ignore bags
	if not diff1 then return end
	-- Apply Base Stat Mods
	if profileDB.enableStatMods then
		sum["STR"] = (sum["STR"] or 0) * StatLogic:GetStatMod("MOD_STR")
		sum["AGI"] = (sum["AGI"] or 0) * StatLogic:GetStatMod("MOD_AGI")
		sum["STA"] = (sum["STA"] or 0) * StatLogic:GetStatMod("MOD_STA")
		sum["INT"] = (sum["INT"] or 0) * StatLogic:GetStatMod("MOD_INT")
		sum["SPI"] = (sum["SPI"] or 0) * StatLogic:GetStatMod("MOD_SPI")
		diff1["STR"] = (diff1["STR"] or 0) * StatLogic:GetStatMod("MOD_STR")
		diff1["AGI"] = (diff1["AGI"] or 0) * StatLogic:GetStatMod("MOD_AGI")
		diff1["STA"] = (diff1["STA"] or 0) * StatLogic:GetStatMod("MOD_STA")
		diff1["INT"] = (diff1["INT"] or 0) * StatLogic:GetStatMod("MOD_INT")
		diff1["SPI"] = (diff1["SPI"] or 0) * StatLogic:GetStatMod("MOD_SPI")
		if diff2 then
			diff2["STR"] = (diff2["STR"] or 0) * StatLogic:GetStatMod("MOD_STR")
			diff2["AGI"] = (diff2["AGI"] or 0) * StatLogic:GetStatMod("MOD_AGI")
			diff2["STA"] = (diff2["STA"] or 0) * StatLogic:GetStatMod("MOD_STA")
			diff2["INT"] = (diff2["INT"] or 0) * StatLogic:GetStatMod("MOD_INT")
			diff2["SPI"] = (diff2["SPI"] or 0) * StatLogic:GetStatMod("MOD_SPI")
		end
	end
	-- Summary Table
	local summary = {}
	local calcSum = profileDB.calcSum
	local calcDiff = profileDB.calcDiff
	local showZeroValueStat = profileDB.showZeroValueStat
	-- Base Stats
	-- Health - HEALTH, STA
	if classDB.sumHP then
		local s, d1, d2
		if calcSum then
			s = ((sum["HEALTH"] or 0) + (sum["STA"] * 10)) * StatLogic:GetStatMod("MOD_HEALTH")
		end
		if calcDiff then
			d1 = ((diff1["HEALTH"] or 0) + (diff1["STA"] * 10)) * StatLogic:GetStatMod("MOD_HEALTH")
			if diff2 then
				d2 = ((diff2["HEALTH"] or 0) + (diff2["STA"] * 10)) * StatLogic:GetStatMod("MOD_HEALTH")
			end
		end
		tinsert(summary, {"HEALTH", s, d1, d2})
	end
	-- Mana - MANA, INT
	if classDB.sumMP then
		local s, d1, d2
		if calcSum then
			s = ((sum["MANA"] or 0) + (sum["INT"] * 15)) * StatLogic:GetStatMod("MOD_MANA")
		end
		if calcDiff then
			d1 = ((diff1["MANA"] or 0) + (diff1["INT"] * 15)) * StatLogic:GetStatMod("MOD_MANA")
			if diff2 then
				d2 = ((diff2["MANA"] or 0) + (diff2["INT"] * 15)) * StatLogic:GetStatMod("MOD_MANA")
			end
		end
		tinsert(summary, {"MANA", s, d1, d2})
	end
	-- Attack Power - AP, STR, AGI
	if classDB.sumAP then
		local s, d1, d2
		if calcSum then
			s = ((sum["AP"] or 0) + (sum["STR"] * StatLogic:GetAPPerStr(class)) + (sum["AGI"] * StatLogic:GetAPPerAgi(class))) * StatLogic:GetStatMod("MOD_AP")
		end
		if calcDiff then
			d1 = ((diff1["AP"] or 0) + (diff1["STR"] * StatLogic:GetAPPerStr(class)) + (diff1["AGI"] * StatLogic:GetAPPerAgi(class))) * StatLogic:GetStatMod("MOD_AP")
			if diff2 then
				d2 = ((diff2["AP"] or 0) + (diff2["STR"] * StatLogic:GetAPPerStr(class)) + (diff2["AGI"] * StatLogic:GetAPPerAgi(class))) * StatLogic:GetStatMod("MOD_AP")
			end
		end
		tinsert(summary, {"AP", s, d1, d2})
	end
	-- Ranged Attack Power - RANGED_AP, AP, AGI, INT
	if classDB.sumRAP then
		local s, d1, d2
		if calcSum then
			s = ((sum["RANGED_AP"] or 0) + (sum["AP"] or 0) + (sum["AGI"] * StatLogic:GetRAPPerAgi(class)) + (sum["INT"] * StatLogic:GetStatMod("ADD_RANGED_AP_MOD_INT"))) * StatLogic:GetStatMod("MOD_RANGED_AP")
		end
		if calcDiff then
			d1 = ((diff1["RANGED_AP"] or 0) + (diff1["AP"] or 0) + (diff1["AGI"] * StatLogic:GetRAPPerAgi(class)) + (diff1["INT"] * StatLogic:GetStatMod("ADD_RANGED_AP_MOD_INT"))) * StatLogic:GetStatMod("MOD_RANGED_AP")
			if diff2 then
				d2 = ((diff2["RANGED_AP"] or 0) + (diff2["AP"] or 0) + (diff2["AGI"] * StatLogic:GetRAPPerAgi(class)) + (diff2["INT"] * StatLogic:GetStatMod("ADD_RANGED_AP_MOD_INT"))) * StatLogic:GetStatMod("MOD_RANGED_AP")
			end
		end
		tinsert(summary, {"RANGED_AP", s, d1, d2})
	end
	-- Feral Attack Power - FERAL_AP, AP, STR, AGI
	if classDB.sumFAP then
		local s, d1, d2
		if calcSum then
			s = ((sum["FERAL_AP"] or 0) + (sum["AP"] or 0) + (sum["STR"] * StatLogic:GetAPPerStr(class)) + (sum["AGI"] * StatLogic:GetAPPerAgi(class))) * StatLogic:GetStatMod("MOD_AP")
		end
		if calcDiff then
			d1 = ((diff1["FERAL_AP"] or 0) + (diff1["AP"] or 0) + (diff1["STR"] * StatLogic:GetAPPerStr(class)) + (diff1["AGI"] * StatLogic:GetAPPerAgi(class))) * StatLogic:GetStatMod("MOD_AP")
			if diff2 then
				d2 = ((diff2["FERAL_AP"] or 0) + (diff2["AP"] or 0) + (diff2["STR"] * StatLogic:GetAPPerStr(class)) + (diff2["AGI"] * StatLogic:GetAPPerAgi(class))) * StatLogic:GetStatMod("MOD_AP")
			end
		end
		tinsert(summary, {"FERAL_AP", s, d1, d2})
	end
	-- Spell Damage - SPELL_DMG, STA, INT, SPI
	if classDB.sumSpellDmg then
		local s, d1, d2
		if calcSum then
			s = ((sum["SPELL_DMG"] or 0) + (sum["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (sum["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (sum["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
		end
		if calcDiff then
			d1 = ((diff1["SPELL_DMG"] or 0) + (diff1["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (diff1["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (diff1["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			if diff2 then
				d2 = ((diff2["SPELL_DMG"] or 0) + (diff2["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (diff2["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (diff2["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			end
		end
		tinsert(summary, {"SPELL_DMG", s, d1, d2})
	end
	-- Holy Damage - HOLY_SPELL_DMG, SPELL_DMG, INT, SPI
	if classDB.sumHolyDmg then
		local s, d1, d2
		if calcSum then
			s = ((sum["HOLY_SPELL_DMG"] or 0) + (sum["SPELL_DMG"] or 0) + (sum["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (sum["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (sum["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
		end
		if calcDiff then
			d1 = ((diff1["HOLY_SPELL_DMG"] or 0) + (diff1["SPELL_DMG"] or 0) + (diff1["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (diff1["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (diff1["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			if diff2 then
				d2 = ((diff2["HOLY_SPELL_DMG"] or 0) + (diff2["SPELL_DMG"] or 0) + (diff2["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (diff2["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (diff2["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			end
		end
		tinsert(summary, {"HOLY_SPELL_DMG", s, d1, d2})
	end
	-- Arcane Damage - ARCANE_SPELL_DMG, SPELL_DMG, INT
	if classDB.sumArcaneDmg then
		local s, d1, d2
		if calcSum then
			s = ((sum["ARCANE_SPELL_DMG"] or 0) + (sum["SPELL_DMG"] or 0) + (sum["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (sum["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (sum["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
		end
		if calcDiff then
			d1 = ((diff1["ARCANE_SPELL_DMG"] or 0) + (diff1["SPELL_DMG"] or 0) + (diff1["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (diff1["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (diff1["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			if diff2 then
				d2 = ((diff2["ARCANE_SPELL_DMG"] or 0) + (diff2["SPELL_DMG"] or 0) + (diff2["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (diff2["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (diff2["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			end
		end
		tinsert(summary, {"ARCANE_SPELL_DMG", s, d1, d2})
	end
	-- Fire Damage - FIRE_SPELL_DMG, SPELL_DMG, STA, INT
	if classDB.sumFireDmg then
		local s, d1, d2
		if calcSum then
			s = ((sum["FIRE_SPELL_DMG"] or 0) + (sum["SPELL_DMG"] or 0) + (sum["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (sum["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (sum["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
		end
		if calcDiff then
			d1 = ((diff1["FIRE_SPELL_DMG"] or 0) + (diff1["SPELL_DMG"] or 0) + (diff1["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (diff1["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (diff1["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			if diff2 then
				d2 = ((diff2["FIRE_SPELL_DMG"] or 0) + (diff2["SPELL_DMG"] or 0) + (diff2["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (diff2["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (diff2["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			end
		end
		tinsert(summary, {"FIRE_SPELL_DMG", s, d1, d2})
	end
	-- Nature Damage - NATURE_SPELL_DMG, SPELL_DMG, INT
	if classDB.sumNatureDmg then
		local s, d1, d2
		if calcSum then
			s = ((sum["NATURE_SPELL_DMG"] or 0) + (sum["SPELL_DMG"] or 0) + (sum["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (sum["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (sum["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
		end
		if calcDiff then
			d1 = ((diff1["NATURE_SPELL_DMG"] or 0) + (diff1["SPELL_DMG"] or 0) + (diff1["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (diff1["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (diff1["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			if diff2 then
				d2 = ((diff2["NATURE_SPELL_DMG"] or 0) + (diff2["SPELL_DMG"] or 0) + (diff2["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (diff2["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (diff2["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			end
		end
		tinsert(summary, {"NATURE_SPELL_DMG", s, d1, d2})
	end
	-- Frost Damage - FROST_SPELL_DMG, SPELL_DMG, INT
	if classDB.sumFrostDmg then
		local s, d1, d2
		if calcSum then
			s = ((sum["FROST_SPELL_DMG"] or 0) + (sum["SPELL_DMG"] or 0) + (sum["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (sum["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (sum["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
		end
		if calcDiff then
			d1 = ((diff1["FROST_SPELL_DMG"] or 0) + (diff1["SPELL_DMG"] or 0) + (diff1["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (diff1["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (diff1["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			if diff2 then
				d2 = ((diff2["FROST_SPELL_DMG"] or 0) + (diff2["SPELL_DMG"] or 0) + (diff2["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (diff2["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (diff2["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			end
		end
		tinsert(summary, {"FROST_SPELL_DMG", s, d1, d2})
	end
	-- Shadow Damage - SHADOW_SPELL_DMG, SPELL_DMG, STA, INT, SPI
	if classDB.sumShadowDmg then
		local s, d1, d2
		if calcSum then
			s = ((sum["SHADOW_SPELL_DMG"] or 0) + (sum["SPELL_DMG"] or 0) + (sum["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (sum["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (sum["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
		end
		if calcDiff then
			d1 = ((diff1["SHADOW_SPELL_DMG"] or 0) + (diff1["SPELL_DMG"] or 0) + (diff1["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (diff1["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (diff1["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			if diff2 then
				d2 = ((diff2["SHADOW_SPELL_DMG"] or 0) + (diff2["SPELL_DMG"] or 0) + (diff2["STA"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_STA")) + (diff2["INT"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_INT")) + (diff2["SPI"] * StatLogic:GetStatMod("ADD_SPELL_DMG_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			end
		end
		tinsert(summary, {"SHADOW_SPELL_DMG", s, d1, d2})
	end
	-- Healing - HEAL, STR, INT, SPI
	if classDB.sumHealing then
		local s, d1, d2
		if calcSum then
			s = ((sum["HEAL"] or 0) + (sum["STR"] * StatLogic:GetStatMod("ADD_HEALING_MOD_STR")) + (sum["INT"] * StatLogic:GetStatMod("ADD_HEALING_MOD_INT")) + (sum["SPI"] * StatLogic:GetStatMod("ADD_HEALING_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
		end
		if calcDiff then
			d1 = ((diff1["HEAL"] or 0) + (diff1["STR"] * StatLogic:GetStatMod("ADD_HEALING_MOD_STR")) + (diff1["INT"] * StatLogic:GetStatMod("ADD_HEALING_MOD_INT")) + (diff1["SPI"] * StatLogic:GetStatMod("ADD_HEALING_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			if diff2 then
				d2 = ((diff2["HEAL"] or 0) + (diff2["STR"] * StatLogic:GetStatMod("ADD_HEALING_MOD_STR")) + (diff2["INT"] * StatLogic:GetStatMod("ADD_HEALING_MOD_INT")) + (diff2["SPI"] * StatLogic:GetStatMod("ADD_HEALING_MOD_SPI"))) * StatLogic:GetStatMod("MOD_SPELL_DMG")
			end
		end
		tinsert(summary, {"HEAL", s, d1, d2})
	end
	-- Hit Chance - MELEE_HIT_RATING, WEAPON_RATING
	if classDB.sumHit then
		local s, d1, d2
		if calcSum then
			s = 0
			for id, v in pairs(sum) do
				if strsub(id, -13) == "WEAPON_RATING" then
					s = s + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
					break
				end
			end
			s = s + StatLogic:GetEffectFromRating((sum["MELEE_HIT_RATING"] or 0), "MELEE_HIT_RATING", calcLevel)
		end
		if calcDiff then
			d1 = 0
			for id, v in pairs(diff1) do
				if strsub(id, -13) == "WEAPON_RATING" then
					d1 = d1 + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
					break
				end
			end
			d1 = d1 + StatLogic:GetEffectFromRating((diff1["MELEE_HIT_RATING"] or 0), "MELEE_HIT_RATING", calcLevel)
			if diff2 then
				d2 = 0
				for id, v in pairs(diff2) do
					if strsub(id, -13) == "WEAPON_RATING" then
						d2 = d2 + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
						break
					end
				end
				d2 = d2 + StatLogic:GetEffectFromRating((diff2["MELEE_HIT_RATING"] or 0), "MELEE_HIT_RATING", calcLevel)
			end
		end
		tinsert(summary, {"MELEE_HIT", s, d1, d2, true})
	end
	-- Crit Chance - MELEE_CRIT_RATING, WEAPON_RATING, AGI
	if classDB.sumCrit then
		local s, d1, d2
		if calcSum then
			s = 0
			for id, v in pairs(sum) do
				if strsub(id, -13) == "WEAPON_RATING" then
					s = s + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
					break
				end
			end
			s = s + StatLogic:GetEffectFromRating((sum["MELEE_CRIT_RATING"] or 0), "MELEE_CRIT_RATING", calcLevel)
			s = s + StatLogic:GetCritFromAgi(sum["AGI"], class, calcLevel)
		end
		if calcDiff then
			d1 = 0
			for id, v in pairs(diff1) do
				if strsub(id, -13) == "WEAPON_RATING" then
					d1 = d1 + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
					break
				end
			end
			d1 = d1 + StatLogic:GetEffectFromRating((diff1["MELEE_CRIT_RATING"] or 0), "MELEE_CRIT_RATING", calcLevel)
			d1 = d1 + StatLogic:GetCritFromAgi(diff1["AGI"], class, calcLevel)
			if diff2 then
				d2 = 0
				for id, v in pairs(diff2) do
					if strsub(id, -13) == "WEAPON_RATING" then
						d2 = d2 + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
						break
					end
				end
				d2 = d2 + StatLogic:GetEffectFromRating((diff2["MELEE_CRIT_RATING"] or 0), "MELEE_CRIT_RATING", calcLevel)
				d2 = d2 + StatLogic:GetCritFromAgi(diff2["AGI"], class, calcLevel)
			end
		end
		tinsert(summary, {"MELEE_CRIT", s, d1, d2, true})
	end
	-- Spell Hit Chance - SPELL_HIT_RATING
	if classDB.sumSpellHit then
		local s, d1, d2
		if calcSum then
			s = StatLogic:GetEffectFromRating((sum["SPELL_HIT_RATING"] or 0), "SPELL_HIT_RATING", calcLevel)
		end
		if calcDiff then
			d1 = StatLogic:GetEffectFromRating((diff1["SPELL_HIT_RATING"] or 0), "SPELL_HIT_RATING", calcLevel)
			if diff2 then
				d2 = StatLogic:GetEffectFromRating((diff2["SPELL_HIT_RATING"] or 0), "SPELL_HIT_RATING", calcLevel)
			end
		end
		tinsert(summary, {"SPELL_HIT", s, d1, d2, true})
	end
	-- Spell Crit Chance - SPELL_CRIT_RATING, INT
	if classDB.sumSpellCrit then
		local s, d1, d2
		if calcSum then
			s = StatLogic:GetEffectFromRating((sum["SPELL_CRIT_RATING"] or 0), "SPELL_CRIT_RATING", calcLevel) + StatLogic:GetSpellCritFromInt(sum["INT"], class, calcLevel)
		end
		if calcDiff then
			d1 = StatLogic:GetEffectFromRating((diff1["SPELL_CRIT_RATING"] or 0), "SPELL_CRIT_RATING", calcLevel) + StatLogic:GetSpellCritFromInt(diff1["INT"], class, calcLevel)
			if diff2 then
				d2 = StatLogic:GetEffectFromRating((diff2["SPELL_CRIT_RATING"] or 0), "SPELL_CRIT_RATING", calcLevel) + StatLogic:GetSpellCritFromInt(diff2["INT"], class, calcLevel)
			end
		end
		tinsert(summary, {"SPELL_CRIT", s, d1, d2, true})
	end
	-- Mana Regen - MANA_REG, SPI, INT
	if classDB.sumMP5 then
		local s, d1, d2
		if calcSum then
			s = (sum["MANA_REG"] or 0) + (sum["INT"] * StatLogic:GetStatMod("ADD_MANA_REG_MOD_INT")) + (StatLogic:GetNormalManaRegenFromSpi(sum["SPI"], class) * StatLogic:GetStatMod("ADD_MANA_REG_MOD_NORMAL_MANA_REG"))
		end
		if calcDiff then
			d1 = (diff1["MANA_REG"] or 0) + (diff1["INT"] * StatLogic:GetStatMod("ADD_MANA_REG_MOD_INT")) + (StatLogic:GetNormalManaRegenFromSpi(diff1["SPI"], class) * StatLogic:GetStatMod("ADD_MANA_REG_MOD_NORMAL_MANA_REG"))
			if diff2 then
				d2 = (diff2["MANA_REG"] or 0) + (diff2["INT"] * StatLogic:GetStatMod("ADD_MANA_REG_MOD_INT")) + (StatLogic:GetNormalManaRegenFromSpi(diff2["SPI"], class) * StatLogic:GetStatMod("ADD_MANA_REG_MOD_NORMAL_MANA_REG"))
			end
		end
		tinsert(summary, {"MANA_REG", s, d1, d2})
	end
	-- Health Regen - HEALTH_REG
	if classDB.sumHP5 then
		local s, d1, d2
		if calcSum then
			s = (sum["HEALTH_REG"] or 0)
		end
		if calcDiff then
			d1 = (diff1["HEALTH_REG"] or 0)
			if diff2 then
				d2 = (diff2["HEALTH_REG"] or 0)
			end
		end
		tinsert(summary, {"HEALTH_REG", s, d1, d2})
	end
	-- Mana Regen while Not casting - SPI
	if classDB.sumMP5NC then
		local s, d1, d2
		if calcSum then
			s = StatLogic:GetNormalManaRegenFromSpi(sum["SPI"], class)
		end
		if calcDiff then
			d1 = StatLogic:GetNormalManaRegenFromSpi(diff1["SPI"], class)
			if diff2 then
				d2 = StatLogic:GetNormalManaRegenFromSpi(diff2["SPI"], class)
			end
		end
		tinsert(summary, {"MANA_REG_NOT_CASTING", s, d1, d2})
	end
	-- Health Regen while Out of Combat - SPI
	if classDB.sumHP5OC then
		local s, d1, d2
		if calcSum then
			s = StatLogic:GetHealthRegenFromSpi(sum["SPI"], class)
		end
		if calcDiff then
			d1 = StatLogic:GetHealthRegenFromSpi(diff1["SPI"], class)
			if diff2 then
				d2 = StatLogic:GetHealthRegenFromSpi(diff2["SPI"], class)
			end
		end
		tinsert(summary, {"HEALTH_REG_OUT_OF_COMBAT", s, d1, d2})
	end
	-- Armor - ARMOR, ARMOR_BONUS, AGI, INT
	if classDB.sumArmor then
		local s, d1, d2
		if calcSum then
			s = (sum["ARMOR"] or 0) * StatLogic:GetStatMod("MOD_ARMOR") + (sum["ARMOR_BONUS"] or 0) + ((sum["AGI"] or 0) * 2) + ((sum["INT"] or 0) * StatLogic:GetStatMod("ADD_ARMOR_MOD_INT"))
		end
		if calcDiff then
			d1 = (diff1["ARMOR"] or 0) * StatLogic:GetStatMod("MOD_ARMOR") + (diff1["ARMOR_BONUS"] or 0) + ((diff1["AGI"] or 0) * 2) + ((diff1["INT"] or 0) * StatLogic:GetStatMod("ADD_ARMOR_MOD_INT"))
			if diff2 then
				d2 = (diff2["ARMOR"] or 0) * StatLogic:GetStatMod("MOD_ARMOR") + (diff2["ARMOR_BONUS"] or 0) + ((diff2["AGI"] or 0) * 2) + ((diff2["INT"] or 0) * StatLogic:GetStatMod("ADD_ARMOR_MOD_INT"))
			end
		end
		tinsert(summary, {"ARMOR", s, d1, d2})
	end
	-- Block Value - BLOCK_VALUE, STR
	if classDB.sumBlockValue then
		local s, d1, d2
		if calcSum then
			s = (sum["BLOCK_VALUE"] or 0) * StatLogic:GetStatMod("MOD_BLOCK_VALUE") + ((sum["STR"] or 0) * StatLogic:GetBlockValuePerStr(class))
		end
		if calcDiff then
			d1 = (diff1["BLOCK_VALUE"] or 0) * StatLogic:GetStatMod("MOD_BLOCK_VALUE") + ((diff1["STR"] or 0) * StatLogic:GetBlockValuePerStr(class))
			if diff2 then
				d2 = (diff2["BLOCK_VALUE"] or 0) * StatLogic:GetStatMod("MOD_BLOCK_VALUE") + ((diff2["STR"] or 0) * StatLogic:GetBlockValuePerStr(class))
			end
		end
		tinsert(summary, {"BLOCK_VALUE", s, d1, d2})
	end
	-- Dodge Chance - DODGE_RATING, DEFENSE_RATING, AGI
	if classDB.sumDodge then
		local s, d1, d2
		if calcSum then
			s = StatLogic:GetEffectFromRating((sum["DODGE_RATING"] or 0), "DODGE_RATING", calcLevel) + StatLogic:GetEffectFromRating((sum["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04 + StatLogic:GetDodgeFromAgi(sum["AGI"])
		end
		if calcDiff then
			d1 = StatLogic:GetEffectFromRating((diff1["DODGE_RATING"] or 0), "DODGE_RATING", calcLevel) + StatLogic:GetEffectFromRating((diff1["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04 + StatLogic:GetDodgeFromAgi(diff1["AGI"])
			if diff2 then
				d2 = StatLogic:GetEffectFromRating((diff2["DODGE_RATING"] or 0), "DODGE_RATING", calcLevel) + StatLogic:GetEffectFromRating((diff2["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04 + StatLogic:GetDodgeFromAgi(diff2["AGI"])
			end
		end
		tinsert(summary, {"DODGE", s, d1, d2, true})
	end
	-- Parry Chance - PARRY_RATING, DEFENSE_RATING
	if classDB.sumParry then
		local s, d1, d2
		if calcSum then
			if GetParryChance() == 0 then
				s = 0
			else
				s = StatLogic:GetEffectFromRating((sum["PARRY_RATING"] or 0), "PARRY_RATING", calcLevel) + StatLogic:GetEffectFromRating((sum["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04
			end
		end
		if calcDiff then
			if GetParryChance() == 0 then
				d1 = 0
			else
				d1 = StatLogic:GetEffectFromRating((diff1["PARRY_RATING"] or 0), "PARRY_RATING", calcLevel) + StatLogic:GetEffectFromRating((diff1["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04
			end
			if diff2 then
				if GetParryChance() == 0 then
					d2 = 0
				else
					d2 = StatLogic:GetEffectFromRating((diff2["PARRY_RATING"] or 0), "PARRY_RATING", calcLevel) + StatLogic:GetEffectFromRating((diff2["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04
				end
			end
		end
		tinsert(summary, {"PARRY", s, d1, d2, true})
	end
	-- Block Chance - BLOCK_RATING, DEFENSE_RATING
	if classDB.sumBlock then
		local s, d1, d2
		if calcSum then
			if GetBlockChance() == 0 then
				s = 0
			else
				s = StatLogic:GetEffectFromRating((sum["BLOCK_RATING"] or 0), "BLOCK_RATING", calcLevel) + StatLogic:GetEffectFromRating((sum["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04
			end
		end
		if calcDiff then
			if GetBlockChance() == 0 then
				d1 = 0
			else
				d1 = StatLogic:GetEffectFromRating((diff1["BLOCK_RATING"] or 0), "BLOCK_RATING", calcLevel) + StatLogic:GetEffectFromRating((diff1["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04
			end
			if diff2 then
				if GetBlockChance() == 0 then
					d2 = 0
				else
					d2 = StatLogic:GetEffectFromRating((diff2["BLOCK_RATING"] or 0), "BLOCK_RATING", calcLevel) + StatLogic:GetEffectFromRating((diff2["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04
				end
			end
		end
		tinsert(summary, {"BLOCK", s, d1, d2, true})
	end
	-- Hit Avoidance - DEFENSE_RATING, MELEE_HIT_AVOID_RATING
	if classDB.sumHitAvoid then
		local s, d1, d2
		if calcSum then
			s = StatLogic:GetEffectFromRating((sum["MELEE_HIT_AVOID_RATING"] or 0), "MELEE_HIT_AVOID_RATING", calcLevel) + StatLogic:GetEffectFromRating((sum["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04
		end
		if calcDiff then
			d1 = StatLogic:GetEffectFromRating((diff1["MELEE_HIT_AVOID_RATING"] or 0), "MELEE_HIT_AVOID_RATING", calcLevel) + StatLogic:GetEffectFromRating((diff1["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04
			if diff2 then
				d2 = StatLogic:GetEffectFromRating((diff2["MELEE_HIT_AVOID_RATING"] or 0), "MELEE_HIT_AVOID_RATING", calcLevel) + StatLogic:GetEffectFromRating((diff2["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04
			end
		end
		tinsert(summary, {"MELEE_HIT_AVOID", s, d1, d2, true})
	end
	-- Crit Avoidance - DEFENSE_RATING, RESILIENCE_RATING, MELEE_CRIT_AVOID_RATING
	if classDB.sumCritAvoid then
		local s, d1, d2
		if calcSum then
			s = StatLogic:GetEffectFromRating((sum["MELEE_CRIT_AVOID_RATING"] or 0), "MELEE_CRIT_AVOID_RATING", calcLevel) + StatLogic:GetEffectFromRating((sum["RESILIENCE_RATING"] or 0), "RESILIENCE_RATING", calcLevel) + StatLogic:GetEffectFromRating((sum["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04
		end
		if calcDiff then
			d1 = StatLogic:GetEffectFromRating((diff1["MELEE_CRIT_AVOID_RATING"] or 0), "MELEE_CRIT_AVOID_RATING", calcLevel) + StatLogic:GetEffectFromRating((diff1["RESILIENCE_RATING"] or 0), "RESILIENCE_RATING", calcLevel) + StatLogic:GetEffectFromRating((diff1["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04
			if diff2 then
				d2 = StatLogic:GetEffectFromRating((diff2["MELEE_CRIT_AVOID_RATING"] or 0), "MELEE_CRIT_AVOID_RATING", calcLevel) + StatLogic:GetEffectFromRating((diff2["RESILIENCE_RATING"] or 0), "RESILIENCE_RATING", calcLevel) + StatLogic:GetEffectFromRating((diff2["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel) * 0.04
			end
		end
		tinsert(summary, {"MELEE_CRIT_AVOID", s, d1, d2, true})
	end
	-- Dodge Neglect - WEAPON_RATING
	if classDB.sumDodgeNeglect then
		local s, d1, d2
		if calcSum then
			s = 0
			for id, v in pairs(sum) do
				if strsub(id, -13) == "WEAPON_RATING" then
					s = s + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
					break
				end
			end
		end
		if calcDiff then
			d1 = 0
			for id, v in pairs(diff1) do
				if strsub(id, -13) == "WEAPON_RATING" then
					d1 = d1 + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
					break
				end
			end
			if diff2 then
				d2 = 0
				for id, v in pairs(diff2) do
					if strsub(id, -13) == "WEAPON_RATING" then
						d2 = d2 + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
						break
					end
				end
			end
		end
		tinsert(summary, {"DODGE_NEGLECT", s, d1, d2, true})
	end
	-- Parry Neglect - WEAPON_RATING
	if classDB.sumParryNeglect then
		local s, d1, d2
		if calcSum then
			s = 0
			for id, v in pairs(sum) do
				if strsub(id, -13) == "WEAPON_RATING" then
					s = s + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
					break
				end
			end
		end
		if calcDiff then
			d1 = 0
			for id, v in pairs(diff1) do
				if strsub(id, -13) == "WEAPON_RATING" then
					d1 = d1 + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
					break
				end
			end
			if diff2 then
				d2 = 0
				for id, v in pairs(diff2) do
					if strsub(id, -13) == "WEAPON_RATING" then
						d2 = d2 + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
						break
					end
				end
			end
		end
		tinsert(summary, {"PARRY_NEGLECT", s, d1, d2, true})
	end
	-- Block Neglect - WEAPON_RATING
	if classDB.sumBlockNeglect then
		local s, d1, d2
		if calcSum then
			s = 0
			for id, v in pairs(sum) do
				if strsub(id, -13) == "WEAPON_RATING" then
					s = s + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
					break
				end
			end
		end
		if calcDiff then
			d1 = 0
			for id, v in pairs(diff1) do
				if strsub(id, -13) == "WEAPON_RATING" then
					d1 = d1 + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
					break
				end
			end
			if diff2 then
				d2 = 0
				for id, v in pairs(diff2) do
					if strsub(id, -13) == "WEAPON_RATING" then
						d2 = d2 + StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel) * 0.04
						break
					end
				end
			end
		end
		tinsert(summary, {"BLOCK_NEGLECT", s, d1, d2, true})
	end
	---------------------
	-- Composite Stats --
	---------------------
	-- Strength - STR
	if classDB.sumStr then
		local s, d1, d2
		if calcSum then
			s = sum["STR"]
		end
		if calcDiff then
			d1 = diff1["STR"]
			if diff2 then
				d2 = diff2["STR"]
			end
		end
		tinsert(summary, {"STR", s, d1, d2})
	end
	-- Agility - AGI
	if classDB.sumAgi then
		local s, d1, d2
		if calcSum then
			s = sum["AGI"]
		end
		if calcDiff then
			d1 = diff1["AGI"]
			if diff2 then
				d2 = diff2["AGI"]
			end
		end
		tinsert(summary, {"AGI", s, d1, d2})
	end
	-- Stamina - STA
	if classDB.sumSta then
		local s, d1, d2
		if calcSum then
			s = sum["STA"]
		end
		if calcDiff then
			d1 = diff1["STA"]
			if diff2 then
				d2 = diff2["STA"]
			end
		end
		tinsert(summary, {"STA", s, d1, d2})
	end
	-- Intellect - INT
	if classDB.sumInt then
		local s, d1, d2
		if calcSum then
			s = sum["INT"]
		end
		if calcDiff then
			d1 = diff1["INT"]
			if diff2 then
				d2 = diff2["INT"]
			end
		end
		tinsert(summary, {"INT", s, d1, d2})
	end
	-- Spirit - SPI
	if classDB.sumSpi then
		local s, d1, d2
		if calcSum then
			s = sum["SPI"]
		end
		if calcDiff then
			d1 = diff1["SPI"]
			if diff2 then
				d2 = diff2["SPI"]
			end
		end
		tinsert(summary, {"SPI", s, d1, d2})
	end
	-- Defense - DEFENSE_RATING
	if classDB.sumDefense then
		local s, d1, d2
		if calcSum then
			s = StatLogic:GetEffectFromRating((sum["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel)
		end
		if calcDiff then
			d1 = StatLogic:GetEffectFromRating((diff1["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel)
			if diff2 then
				d2 = StatLogic:GetEffectFromRating((diff2["DEFENSE_RATING"] or 0), "DEFENSE_RATING", calcLevel)
			end
		end
		tinsert(summary, {"DEFENSE", s, d1, d2})
	end
	-- Weapon Skill - WEAPON_RATING
	if classDB.sumWeaponSkill then
		local s, d1, d2
		local weapon = {}
		if calcSum then
			for id, v in pairs(sum) do
				local s, d1, d2
				if strsub(id, -13) == "WEAPON_RATING" then
					s = StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel)
					if calcDiff then
						d1 = StatLogic:GetEffectFromRating((diff1[id] or 0), CR_WEAPON_SKILL, calcLevel)
						if diff2 then
							d2 = StatLogic:GetEffectFromRating((diff2[id] or 0), CR_WEAPON_SKILL, calcLevel)
						end
					end
					if not weapon[id] then
						weapon[id] = true
						tinsert(summary, {strsub(id, 1, -8), s, d1, d2})
					end
				end
			end
		end
		if calcDiff then
			for id, v in pairs(diff1) do
				local s, d1, d2
				s = 0
				if strsub(id, -13) == "WEAPON_RATING" then
					d1 = StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel)
					if diff2 then
						d2 = StatLogic:GetEffectFromRating((diff2[id] or 0), CR_WEAPON_SKILL, calcLevel)
					end
					if not weapon[id] then
						weapon[id] = true
						tinsert(summary, {strsub(id, 1, -8), s, d1, d2})
					end
				end
			end
			if diff2 then
				for id, v in pairs(diff2) do
					local s, d1, d2
					s = 0
					d1 = 0
					if strsub(id, -13) == "WEAPON_RATING" then
						d2 = StatLogic:GetEffectFromRating(v, CR_WEAPON_SKILL, calcLevel)
						if not weapon[id] then
							weapon[id] = true
							tinsert(summary, {strsub(id, 1, -8), s, d1, d2})
						end
					end
				end
			end
		end
	end
	------------------------
	-- Build Output Table --
	local output = {}
	for _, t in ipairs(summary) do
		local n, s, d1, d2, ispercent = t[1], t[2], t[3], t[4], t[5]
		local right, left
		local skip
		if not showZeroValueStat then
			if calcSum and calcDiff then
				if s == 0 and d1 == 0 and ((not d2) or (d2 and d2 == 0)) then
					skip = true
				end
			elseif calcSum then
				if s == 0 then
					skip = true
				end
			elseif calcDiff then
				if d1 == 0 and ((not d2) or (d2 and d2 == 0)) then
					skip = true
				end
			end
		end
		if not skip then
			if calcSum and calcDiff then
				local d = ((s - floor(s)) == 0) and ((d1 - floor(d1)) == 0) and ((not d2) or (d2 and (d2 - floor(d2)) == 0))
				if d then
					s = format("%d", s)
				elseif ispercent then
					s = format("%.2f%%", s)
				else
					s = format("%.1f", s)
				end
				if d then
					d1 = colorNum(format("%+d", d1), d1)
				elseif ispercent then
					d1 = colorNum(format("%+.2f%%", d1), d1)
				else
					d1 = colorNum(format("%+.1f", d1), d1)
				end
				if d2 then
					if d then
						d2 = colorNum(format("%+d", d2), d2)
					elseif ispercent then
						d2 = colorNum(format("%+.2f%%", d2), d2)
					else
						d2 = colorNum(format("%+.1f", d2), d2)
					end
					right = format("%s(%s/%s)", s, d1, d2)
				else
					right = format("%s(%s)", s, d1)
				end
			elseif calcSum then
				if (s - floor(s)) == 0 then
					s = format("%d", s)
				elseif ispercent then
					s = format("%.2f%%", s)
				else
					s = format("%.1f", s)
				end
				right = s
			elseif calcDiff then
				local d = ((d1 - floor(d1)) == 0) and ((not d2) or (d2 and (d2 - floor(d2)) == 0))
				if d then
					d1 = colorNum(format("%+d", d1), d1)
				elseif ispercent then
					d1 = colorNum(format("%+.2f%%", d1), d1)
				else
					d1 = colorNum(format("%+.1f", d1), d1)
				end
				if d2 then
					if d then
						d2 = colorNum(format("%+d", d2), d2)
					elseif ispercent then
						d2 = colorNum(format("%+.2f%%", d2), d2)
					else
						d2 = colorNum(format("%+.1f", d2), d2)
					end
					right = format("%s/%s", d1, d2)
				else
					right = d1
				end
			end
			
			left = StatLogic:GetStatNameFromID(n)
			tinsert(output, {left, right})
		end
	end
	-- Write cache
	cache[id] = output
	-------------------
	-- Write Tooltip --
	if table.maxn(output) == 0 then return end
	tooltip:AddLine(HIGHLIGHT_FONT_COLOR_CODE..L["Stat Summary"]..FONT_COLOR_CODE_CLOSE)
	tooltip:AddTexture("Interface\\AddOns\\RatingBuster\\images\\Sigma")
	for _, o in ipairs(output) do
		tooltip:AddDoubleLine(o[1], o[2])
	end
end


-- RatingBuster:Bench(1000)
---------
-- self:SplitDoJoin("+24 Agility/+4 Stamina, +4 Dodge and +4 Spell Crit/+5 Spirit", {"/", " and ", ","})
-- 1000 times: 0.16 - 0.18 without Compost
-- 1000 times: 0.22 - 0.24 with Compost
---------
-- RatingBuster.ProcessTooltip(ItemRefTooltip, link)
-- 1000 times: 0.31 sec - 0.7.6
-- 1000 times: 0.29 sec - 0.
-- 1000 times: 0.24 sec - 0.8.58.0
---------
-- strjoin 1000000 times: 0.46
-- ..      1000000 times: 0.27
--------------
function RatingBuster:Bench(k)
	local t1 = GetTime()
	local link = GetInventoryItemLink("player", 12)
	for i = 1, k, 1 do
		---------------------------------------------------------------------------
		--self:SplitDoJoin("+24 Agility/+4 Stamina, +4 Dodge and +4 Spell Crit/+5 Spirit", {"/", " and ", ","})
		---------------------------------------------------------------------------
		ItemRefTooltip:SetInventoryItem("player", 12)
		RatingBuster.ProcessTooltip(ItemRefTooltip, link)
		---------------------------------------------------------------------------
		--ItemRefTooltip:SetScript("OnTooltipSetItem", function(frame, ...) RatingBuster:Print("OnTooltipSetItem") end)
		----------------------------------------------------------------------
		--local h = strjoin("", "test", "123")
		--local h = "test".."123"
		--------------------------------------------------------------------------------
	end
	return GetTime() - t1
end
